package OandA::Install;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK
	%config
	$Force_Defaults $Use_RCS $CGI_bin
	);

use Cwd;
use File::Path;
use File::Basename;
use Data::Dumper;

require Exporter;

@ISA = qw(Exporter);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	%config
	machine_questions
	set_perl_path
	get_dirpath
	get_filepath
	input
	read_config_file
	save_config
	Force_Defaults
	Use_RCS
	CGI_bin
	configure
	key_install
	install
);
$VERSION = '0.01';
$Force_Defaults = 0;
$Use_RCS = 0;
$CGI_bin = 0;

=head1 NAME

OandA::Install - Perl extension for common install code

=head1 SYNOPSIS

  use OandA::Install;

=head1 DESCRIPTION

Stub documentation for OandA was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 GLOBAL VARIABLES

=head2 $Force_Defaults

When set, user input is bypassed in favor of the defaults. 
Set by calling Force_Defaults().

=head2 $Use_RCS

Set by calling Use_RCS().

=head2 $CGI_bin

Set by calling CGI_bin().

=head2 %config

Configuration data gathered from the user or derived from same.
Data include:

=over 4

=item XXPerlPathXX - the path to the perl executable

=back

=head1 METHODS

=over 4
=item machine_questions - ask questions
=item set_perl_path(default) - get path to perl

Gather machine specific data, presently limited to the path to the
perl executable. 

set_perl_path will try the usual suspects to come up with
a default if %config has no value.

=cut

sub machine_questions
{
	print <<EOF;


Now I need you to answer a few questions about this machine.

For each question:
 +  The default answer is given in [brackets].
 +  To override the default answer, enter the
      correct answer in the space next to the brackets.
 +  To accept the default answer, enter a blank line.
 

EOF
	$config{XXPerlPathXX} = set_perl_path($config{XXPerlPathXX});
}

sub set_perl_path
{
	my $PerlPath = shift;
	my $default = $^X;
	unless ($default =~ m:^/:)
	{
		foreach (reverse split(/:/, $ENV{PATH}))
		{
			next unless m:/^/:;
			next unless -x "$_/$default";
			$default = "$_/$default";
			last;
		}
	}
	while (1)
	{
		$PerlPath ||= $default;
		$PerlPath = join("/", cwd, $PerlPath) unless $PerlPath =~ m:^/:;
		print <<EOF;
What is the full pathname of the 'Perl' executable on this machine?

EOF
		print "[$PerlPath] ";
		$PerlPath = input($PerlPath);
		$PerlPath = join("/", cwd, $PerlPath) unless $PerlPath =~ m:^/:;
		return $PerlPath if -e $PerlPath && -f $PerlPath && -x $PerlPath;
		print "Sorry, '$PerlPath' does not exist.\n" unless -e $PerlPath;
		print "Sorry, '$PerlPath' is not a normal file.\n" unless -f $PerlPath;
		print "Sorry, '$PerlPath' is not executable.\n" unless -x $PerlPath;
	}
}

=item get_dirpath(configtag, label, default, docroot)
=item get_filepath(configtab, label, default, docroot)

=cut

sub get_dirpath
{
	my ($tag, $what, $default, $root) = @_;
	#print STDERR Dumper(@_);
	my $path = $config{$tag} || $default;
	while (1)
	{
		$path ||= $config{$tag} || $default;
		$path = join('/', cwd, $path) unless $path =~ m:^/:;
		print "\nLocal directory for $what:\n[$path] ";
		$path = input($path);
		$path = join('/', cwd, $path) unless $path =~ m:^/:;
		if (-e $path && !-d $path)
		{
			print "Sorry, '$path' exists and is not a directory.\n";
			print "Try again...\n";
			next;
		}
		last if index($path, $root) == 0;
		print "Sorry, '$path' must be under '$root'.\n";
		print "Try again...\n";
	}
	$config{$tag} = $path;
	return $path;
}
sub get_filepath
{
	my ($tag, $what, $default, $root) = @_;
	my $path = $config{$tag} || $default;
	while (1)
	{
		$path ||= $config{$tag} || $default;
		$path = join('/', cwd, $path) unless $path =~ m:^/:;
		print "\nLocal install-path of $what:\n[$path] ";
		$path = input($path);
		$path = join('/', cwd, $path) unless $path =~ m:^/:;
		if (-e $path && !-f $path)
		{
			print "Sorry, '$path' exists and is not a normal file.\n";
			print "Try again...\n";
			next;
		}
		last if index($path, $root) == 0;
		print "Sorry, '$path' must be under '$root'.\n";
		print "Try again...\n";
	}
	$config{$tag} = $path;
	return $path;
}

=item input(default) - get user input with default specified

If $Force_Defaults is set, the default will be returned. If no input is
given, the default will be returned.

=cut

sub input
{
	my $default = shift || '';
	my $in = '';
	if ($Force_Defaults)
	{
		print "\n";
	}
	else
	{
		$in = <STDIN>;
		chomp $in;
		$in =~ s/\s//g;
	}
	return $in eq '' ? $default : $in;
}

=item read_config_file(config_file)
=item save_config(configure_file)

Populate %config from the config_file or save contents of %config to
config_file.

=cut

sub read_config_file
{
	my $conf_file = shift || return;
	return unless -r $conf_file;
	require "$conf_file";
}
sub save_config
{
	my $conf_file = shift;
	local $config{XXHelpersXX} = '';
	unless (open CONF, ">$conf_file")
	{
		print "Unable to save configuration to '$conf_file'.\n";
		return 0;
	}
	print CONF '%config = (', "\n";
	print CONF join("\n", map "\t$_ => '$config{$_}',", sort keys %config), "\n";
	print CONF ");\n1;\n";
	close CONF;
	return 1;
}

=item Force_Defaults(bool)
=item Use_RCS(bool)
=item CGI_bin(bool)

These set the corresponding variables. 

=cut

sub Force_Defaults { @_ ? $Force_Defaults = shift || 0 : $Force_Defaults; }
sub Use_RCS { @_ ?$Use_RCS = shift || 0 : $Use_RCS; }
sub CGI_bin { @_ ? $CGI_bin= shift || 0 : $CGI_bin; }

=item configure(text, label)

Applies all substitutions in %config to text and returns the modified text.
The label is used for display purposes only.

=cut

sub configure
{
	my ($text, $label) = @_;
	print "Configuring $label.\n";
	while (my ($tag, $value) = each %config)
	{
		$text =~ s/$tag/$value/g;
	}
	return $text;
}

=item install(text, path, mode, label)
=item key_install(text, tab, mode, label)

Install the text into the file at path or $config{tag} after making
substitutions.

=cut

sub key_install
{
	my ($text, $tag, $mode, $label) = @_;
	install($text, $config{"XX${tag}PathXX"}, $mode, $label);
}
sub install
{
	my ($text, $path, $mode, $label) = @_;
	my $pmode = sprintf("%o",$mode);
	$text = configure($text, $label);
	print "Installing $label at\n '$path'\nwith mode $pmode.\n";
	if (-e $path)
	{
		if (-w $path)
		{
			print "Is it okay to overwrite existing file\n '$path'? [", $Force_Defaults ? 'y' : 'n', "] ";
			die "Unable to complete installation" unless input($Force_Defaults ? 'y' : 'n') =~ /^y/;
		}
		elsif ($Use_RCS)
		{
			print "Checking out '$path'.\n";
			system('co', '-l', $path);
			die if $?;
		}
		else
		{
			print "Is it okay to delete existing file\n '$path' first? [", $Force_Defaults ? 'y' : 'n', "] ";
			die "Unable to complete installation" unless input($Force_Defaults ? 'y' : 'n') =~ /^y/;
			unlink $path or die "Cannot delete '$path': $!";
		}
	}
	mkpath(dirname($path), 0, 0755);
	
	open(OUT, ">$path") or die "Cannot open '$path' for write in install: $!";
	print OUT $text;
	close(OUT) or die "Cannot close '$path' in install: $!";
	chmod $mode, $path or die "Cannot chmod '$path': $!";
	if ($Use_RCS)
	{
		print "Checking '$path' back in.\n";
		system('ci', '-u', $path);
		die if $?;
	}
}

=back

=head1 AUTHOR

Michael Houghton, herveus@radix.net

Copyright 2000 Michael Houghton, all rights reserved.

Based on previous work by Stephen Gold.

This is free software and may be used under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).

=cut

1;
